%{

(* This program is free software.  See LICENSE for more information *)

let parse_error msg =
print_string msg;;

open Smt_tables;;
open Smt_helper_logic;;

%}
%token <string> NUMERAL_TOK
%token <string> VAR_TOK
%token <string> SYM_TOK
%token <string> STRING_TOK
%token <string> AR_SYMB
%token <string> USER_VAL_TOK
%token TRUE_TOK
%token FALSE_TOK
%token ITE_TOK
%token NOT_TOK
%token IMPLIES_TOK
%token IF_THEN_ELSE_TOK
%token AND_TOK
%token OR_TOK
%token XOR_TOK
%token IFF_TOK
%token EXISTS_TOK
%token FORALL_TOK
%token LET_TOK
%token FLET_TOK
%token NOTES_TOK
%token SORTS_TOK
%token FUNS_TOK
%token PREDS_TOK
%token EXTENSIONS_TOK
%token DEFINITION_TOK
%token AXIOMS_TOK
%token LOGIC_TOK
%token COLON_TOK
%token LPAREN_TOK
%token RPAREN_TOK
%token SAT_TOK
%token UNSAT_TOK
%token UNKNOWN_TOK
%token THEORY_TOK
%token ASSUMPTION_TOK
%token FORMULA_TOK
%token STATUS_TOK
%token BENCHMARK_TOK
%token EXTRASORTS_TOK
%token EXTRAFUNS_TOK
%token EXTRAPREDS_TOK
%token LANGUAGE_TOK
%token DOLLAR_TOK
%token EQUALS_TOK
%token DISTINCT_TOK
%token SEMICOLON_TOK
%token EOF_TOK
%token THEORY_TOK

%start logic
%type <Smt_helper_logic.logic> logic

%%




logic:
    LPAREN_TOK LOGIC_TOK logic_name logic_attributes RPAREN_TOK
    {

      	let b = LOGIC($3,$4) in
      	  if not (Hashtbl.mem symbol_table "theory") then 
            raise (Validation_error "Theory not defined.")
          else 
      	    if not (Hashtbl.mem symbol_table "language") then 
              raise (Validation_error "Language not defined.")
            else 
            (
              b;
            )
    }

logic_name:
    SYM_TOK 
    { 
    	Printf.printf "Logic_name:%s\n" $1;
 		NAME($1)
    }


logic_attributes:
    logic_attribute logic_attributes {$1::$2}
  | logic_attribute {[$1]}

logic_attribute:
  
    COLON_TOK THEORY_TOK SYM_TOK
    {
      if Hashtbl.mem symbol_table "theory" then 
        raise (Validation_error "More than one theory definition found.")
      else (
      	Hashtbl.replace symbol_table "theory" "exists";
      	if ($3 = "Int_Arrays") then
			Hashtbl.replace sorts "_numeralsort_" "Int"      	
      	else
      	if ($3 = "Ints") then
			Hashtbl.replace sorts "_numeralsort_" "Int"      	      	
      	else
      	if ($3 = "Reals_Ints") then
			Hashtbl.replace sorts "_numeralsort_" "Real"      	
      	else
      	if ($3 = "Reals") then
			Hashtbl.replace sorts "_numeralsort_" "Real"      	
      	else
      		Hashtbl.replace sorts "_numeralsort_" "UNKNOWN"
      	;

		let parse_theory filename = 
			let in_ch = open_in filename in
			let lexbuf = Lexing.from_channel in_ch in
			Printf.printf "Parsing Theory File: %s\n" filename;
			let theory = Smt_parser_theory.theory Smt_lexer_theory.token lexbuf in
			close_in in_ch;
			let axioms = Smt_helper_theory.get_axioms_from_theory theory in
			let env_sorts = Hashtbl.find Smt_tables.environment "env" in
			let env = if ((List.length axioms) = 0) then
					[]
				else
					List.map (Smt_helper_theory.check_formula env_sorts) (List.hd axioms)			
				in
			theory in
		  	parse_theory (theories_dir^$3^theory_file_suffix);		  
		  	THEORY($3)
		)
	}
  | COLON_TOK LANGUAGE_TOK STRING_TOK 
    {
      if Hashtbl.mem symbol_table "language" then 
        raise (Validation_error "More than one language definition found.")
      else (
      	Hashtbl.replace symbol_table "language" "exists";
      	
  		LANGUAGE($3)
  		)
  	}
  | COLON_TOK EXTENSIONS_TOK STRING_TOK {EXTENSIONS($3)}
  | COLON_TOK NOTES_TOK STRING_TOK {NOTES($3)}
  | annotation {ANNOTATION($1)}

annotation:
    attribute 
    {
      (* returns the attribute's name and None because it doesn't have a corresponding value *)
      ($1,None)
    }
  | attribute user_value 
    {
      (* returns the attribute's name along with its corresponding value*)
      ($1,Some $2)
    }

    
    

annotations:
    annotation annotations 
    { 
      (*  returns a list of annotations*)
      ($1::$2)
    }
  | annotation 
    {
      (* returns a singleton list of annotations *)
      [$1]
    }
  
annotation:
    attribute 
    {
      (* returns the attribute's name and None because it doesn't have a corresponding value *)
      ($1,None)
    }
  | attribute user_value 
    {
      (* returns the attribute's name along with its corresponding value*)
      ($1,Some $2)
    }

user_value:
    USER_VAL_TOK
    {
    	$1
    }



attribute:
    COLON_TOK SYM_TOK { ":"^ $2}

fvar:
    DOLLAR_TOK SYM_TOK { "$" ^ $2}
    



    